#include <asf.h>
#include "conf_application.h"
#include "delay.h"
#include "gfx_mono_text.h"
#include "gfx_mono_generic.h"
#include "stdio.h"
#include "avr/eeprom.h"
#include "avr/interrupt.h"
#include "avr/sleep.h"
#include "twim.h"
#include "tc.h"

#include "vibration_actuators.h"
#include "force_sensors.h"
#include "magnetic_encoders.h"
#include "buttons.h"

#define LED0_ON			gpio_set_pin_low(LED0)
#define LED0_OFF		gpio_set_pin_high(LED0)
#define LED1_ON			gpio_set_pin_low(LED1)
#define LED1_OFF		gpio_set_pin_high(LED1)
#define LED2_ON			gpio_set_pin_low(LED2)
#define LED2_OFF		gpio_set_pin_high(LED2)
#define LED3_ON			gpio_set_pin_low(LED3)
#define LED3_OFF		gpio_set_pin_high(LED3)

#define CLEAR_SCREEN	gfx_mono_generic_draw_filled_rect(0,0,130,30,0)
#define DISPLAY_RAW_VAL 0

#define NO_OF_ACTUATOR		3
#define NO_OF_ENCODERS		4

#define DEFORMATION_THRESHOLD 5
#define DEFORMATION_SPEED	4
#define PULSE_DURATION		10

static volatile uint8_t read_fsr				= 0;
static volatile uint8_t read_encoder			= 0;
static volatile uint8_t update_pulse_counters	= 0;
static volatile uint8_t print_stuff				= 0;

static void timer_light_isr(void){
	print_stuff = 1;
	gpio_toggle_pin(LED2);
	tc_clear_overflow(&TCC1);
}

static void timer_fsr_isr(void){
	read_fsr = 1;
	tc_clear_overflow(&TCE0);
}

static void timer_enc_isr(void){
	read_encoder = 1;
	tc_clear_overflow(&TCE1);

}

static void timer_pulse_counter_isr(void){
		update_pulse_counters = 1;
		tc_clear_overflow(&TCF0);
}

static void timer_init(void){
	// Set timer to toggle the red LED every second to indicate that the program is running
	tc_enable(&TCC1);
	tc_set_overflow_interrupt_callback(&TCC1,timer_light_isr);
	tc_set_wgm(&TCC1,TC_WG_NORMAL);
	tc_write_period(&TCC1,23437); //(
	tc_set_overflow_interrupt_level(&TCC1,TC_INT_LVL_LO);
	tc_write_clock_source(&TCC1,TC_CLKSEL_DIV1024_gc);

	// Set timer to wake from sleep every 1ms to get a new FSR reading.
	tc_enable(&TCE0);
	tc_set_overflow_interrupt_callback(&TCE0,timer_fsr_isr);
	tc_set_wgm(&TCE0,TC_WG_NORMAL);
	tc_write_period(&TCE0,24000); //(
	tc_set_overflow_interrupt_level(&TCE0,TC_INT_LVL_LO);
	tc_write_clock_source(&TCE0,TC_CLKSEL_DIV1_gc);

	// Set timer every 100ms to get a new encoder reading
	tc_enable(&TCE1);
	tc_set_overflow_interrupt_callback(&TCE1,timer_enc_isr);
	tc_set_wgm(&TCE1,TC_WG_NORMAL);
	tc_write_period(&TCE1,18750); //(
	tc_set_overflow_interrupt_level(&TCE1,TC_INT_LVL_LO);
	tc_write_clock_source(&TCE1,TC_CLKSEL_DIV256_gc);
	
	// Set timer every 10ms to get a better resolution for the pulse counter
	tc_enable(&TCF0);
	tc_set_overflow_interrupt_callback(&TCF0,timer_pulse_counter_isr);
	tc_set_wgm(&TCF0,TC_WG_NORMAL);
	tc_write_period(&TCF0,3750); //(
	tc_set_overflow_interrupt_level(&TCF0,TC_INT_LVL_LO);
	tc_write_clock_source(&TCF0,TC_CLKSEL_DIV64_gc);

}

static void system_init(void){
	cli();
	board_init();
	pmic_init();
	pmic_set_scheduling(PMIC_SCH_ROUND_ROBIN);
	sysclk_init();
	delay_init();
	gfx_mono_init();
	buttons_init();
	irq_initialize_vectors();

	set_sleep_mode(SLEEP_MODE_IDLE);
	timer_init();

	sei();
}

static void haptic_feedback_state_machine_speed_based(void){
	//Force readings as discrete levels
	//Angle readings in degrees
	//Angle speed in degrees / ds (deci second, i.e 10^-1, the same as the polling frequency of the encoders.)
	
	uint8_t force_reading[NO_OF_SENSORS]			= {0};
	uint8_t force_reading_previous[NO_OF_SENSORS]	= {0};
	int16_t angle_reading[NO_OF_ENCODERS]			= {0};
	int16_t angle_reading_previous[NO_OF_ENCODERS]	= {0};
	int16_t angle_speed[NO_OF_ENCODERS]				= {0};
	uint8_t deformation[NO_OF_ENCODERS/2]			= {0};


	uint8_t contact_pulse_counter[NO_OF_SENSORS]	= {0};

	uint8_t contact = 0;

	magnetic_encoders_enable();

	while(1){
		sleep_mode();
		//Force reading
		if (read_fsr){
			read_fsr = 0;
			force_sensors_read_force_levels_with_hysteresis(force_reading);

			contact = force_reading[0] || force_reading[1] || force_reading[2];
			if (contact){
				LED1_ON;
				}else{
				LED1_OFF;
			}
			for (uint8_t sensor_no = 0; sensor_no < NO_OF_SENSORS;sensor_no++){
				if ((force_reading[sensor_no] > 0) && !force_reading_previous[sensor_no]){
					contact_pulse_counter[sensor_no] = 1;
				}
				force_reading_previous[sensor_no] = force_reading[sensor_no];
			}
		}

		//Angle and angle speed reading
		if(contact && read_encoder){
			read_encoder = 0;
			for (uint8_t encoder_no = 0; encoder_no < NO_OF_ENCODERS; encoder_no++){
				angle_reading_previous[encoder_no] = angle_reading[encoder_no];
			}

			magnetic_encoders_read_angle(angle_reading);

			for (uint8_t encoder_no = 0; encoder_no < NO_OF_ENCODERS; encoder_no++){
				angle_speed[encoder_no] = (angle_reading[encoder_no] - angle_reading_previous[encoder_no]);

				//The distal thumb encoder is mounted such that the max angle is in the open position.
				if (encoder_no == 1){
					angle_speed[encoder_no] = -angle_speed[encoder_no];
				}
				//Only interested in closing speeds
				if(angle_speed[encoder_no]<0){
					angle_speed[encoder_no] = 0;
				}
			}

			//Deformation detection
			if (force_reading[2] && force_reading[0]){
				deformation[0] =  ( (angle_speed[0]>DEFORMATION_SPEED) || (angle_speed[1] > DEFORMATION_SPEED) || (angle_speed[2] > DEFORMATION_SPEED) || (angle_speed[3] > DEFORMATION_SPEED));
				}else{
				deformation[0] = 0;
			}
			if (force_reading[2] && force_reading[1]){
				deformation[1] = ((angle_speed[0]>DEFORMATION_SPEED) || (angle_speed[1]> DEFORMATION_SPEED));
				}else{
				deformation[1] = 0;
			}

			if (deformation[0] || deformation[1]){
				for (uint8_t actuator_no = 0; actuator_no < NO_OF_ACTUATOR; actuator_no++){
					vibration_actuator_set_amplitude_level(actuator_no,4);
				}
			}else{
				for (uint8_t actuator_no = 0; actuator_no < NO_OF_ACTUATOR; actuator_no++){
					if (!contact_pulse_counter[actuator_no]){
						vibration_actuator_set_amplitude_level(actuator_no,0);
					}
				}
			}

		}
		if (deformation[0]){
			LED0_ON;
			}else{
			LED0_OFF;
		}

		if (deformation[1]){
			LED2_ON;
			}else{
			LED2_OFF;
		}


		//Pulse Control
		if (update_pulse_counters){
			update_pulse_counters = 0;

			for (uint8_t actuator_no = 0; actuator_no <NO_OF_ACTUATORS;actuator_no++){
				if (contact_pulse_counter[actuator_no]){
					vibration_actuator_set_amplitude_level(actuator_no,4);
					contact_pulse_counter[actuator_no]++;
				}

				if (contact_pulse_counter[actuator_no] > PULSE_DURATION){
					contact_pulse_counter[actuator_no] = 0;
					vibration_actuator_set_amplitude_level(actuator_no,0);
				}
			}
		}
	}
}

int main(void){

	system_init();					//Initializes the board and controller as well as timers.
	uint8_t calibrate;
	gfx_mono_draw_string("SW0 to calibrate",0,0,&sysfont);
	gfx_mono_draw_string("SW1 to skip",0,10,&sysfont);

	while(1){
		if (button_switched[0]){
			button_switched[0] = 0;
			calibrate = 1;
			CLEAR_SCREEN;
			break;
		}if (button_switched[1]){
			button_switched[1] = 0;
			calibrate = 0;
			CLEAR_SCREEN;
			break;
		}
		delay_ms(10);
	}

	force_sensors_init(calibrate);
	magnetic_encoders_init();
	magnetic_encoders_enable();
	vibration_actuator_init();		//Initializes the I2C /TWI module as well as running the auto calibration for the actuators.

	gfx_mono_draw_string("Start up complete!",0,0,&sysfont);
	gfx_mono_draw_string("Press SW0",0,20,&sysfont);

	while(!button_switched[0]);
	button_switched[0] = 0;
	CLEAR_SCREEN;
	
	haptic_feedback_state_machine_speed_based();

	vibration_actuator_all_off();
	magnetic_encoders_disable();
	return 1;
}
